@@ -4,6 +4,7 @@ stores model information in the :class:`cellmlmanip.model.Model` class. MathML e
 is handled by RDFLib.
 """
 import itertools
+import logging
 import os
 from collections import deque
 from enum import Enum
@@ -14,6 +15,9 @@ from lxml import etree
 from cellmlmanip.model import SYMPY_SYMBOL_DELIMITER, Model
 
 
+logger = logging.getLogger(__name__)
+
+
 UNIT_PREFIXES = {
     'yocto': 1e-24,
     'zepto': 1e-21,
@@ -39,6 +43,17 @@ UNIT_PREFIXES = {
 }
 
 
+# Work around for sympy issue dealing with relationals in equations that er passed to Piecewise
+# see https://github.com/sympy/sympy/issues/24086
+# and https://github.com/ModellingWebLab/cellmlmanip/issues/350
+sympy.Eq.is_Boolean = True
+sympy.Ne.is_Boolean = True
+sympy.Ge.is_Boolean = True
+sympy.Le.is_Boolean = True
+sympy.Gt.is_Boolean = True
+sympy.Lt.is_Boolean = True
+
+
 class XmlNs(Enum):
     """Namespaces in CellML documents"""
     CELLML = 'http://www.cellml.org/cellml/1.0#'
@@ -122,6 +137,16 @@ class Parser(object):
 
         # <model> root node - initialise the model object
         model_xml = tree.getroot()
+
+        # Raise error if component units are defined
+        units_in_comp_xpath = with_ns(XmlNs.CELLML, 'component') + '/' + with_ns(XmlNs.CELLML, 'units')
+        units_in_components = model_xml.findall(units_in_comp_xpath)
+        if len(units_in_components) != 0:
+            msg = 'Defining units inside components is not supported (found in component'
+            msg += 's ' if len(units_in_components) > 1 else ' '
+            msg += ', '.join([e.getparent().get('name') for e in units_in_components]) + ').'
+            raise ValueError(msg)
+
         self.model = Model(model_xml.get('name'),
                            model_xml.get(with_ns(XmlNs.CMETA, 'id')),
                            unit_store=unit_store)
@@ -197,6 +222,8 @@ class Parser(object):
             # unit is defined in terms of known units - ok to add to model
             if add_now:
                 definition = self._make_pint_unit_definition(unit_name, unit_elements)
+                if self.model.units.is_defined(unit_name):
+                    raise ValueError(f'Duplicate unit definition {unit_name}, unit names need to be unique!')
                 self.model.units.add_unit(unit_name, definition)
                 units_found.add(unit_name)
                 iteration = 0
@@ -242,6 +269,10 @@ class Parser(object):
             if 'multiplier' in unit_element:
                 expr = '(%s * %s)' % (unit_element['multiplier'], expr)
 
+            if 'offset' in unit_element and (not unit_element['offset'].strip().isnumeric() or
+                                             int(unit_element['offset']) != 0):
+                raise ValueError('Offsets in units are not supported!')
+
             # Collect/add this particular <unit> definition
             full_unit_expr.append(expr)
 
@@ -262,6 +293,8 @@ class Parser(object):
         for element in component_elements:
             # component are only kept in parser to resolve relationships and connections
             name = element.get('name')
+            if name in self.components:
+                raise ValueError(f'Duplicate component name {name}, component names must be unique!')
             self.components[name] = _Component(name)
 
             # process the <variable> tags in this component
@@ -270,12 +303,6 @@ class Parser(object):
             # to speed up parsing, we store the maths to add to perform adding later when we know how variables connect
             component_variables.append((element, variable_to_symbol))
 
-            # Raise error if component units are defined
-            component_units = element.findall(with_ns(XmlNs.CELLML, 'units'))
-            if component_units:
-                raise ValueError(
-                    'Defining units inside components is not supported (found in component ' + name + ').')
-
             # Raise error if reactions are defined
             reactions = element.findall(with_ns(XmlNs.CELLML, 'reaction'))
             if reactions:
@@ -423,6 +450,10 @@ class Parser(object):
             comp_1, comp_2 = (map_components.attrib.get('component_1'),
                               map_components.attrib.get('component_2'))
 
+            if comp_1 not in self.components:
+                raise ValueError(f'Cannot connect components that do not exist: {comp_1}!')
+            if comp_2 not in self.components:
+                raise ValueError(f'Cannot connect components that do not exist: {comp_2}!')
             # go through all tags
             for child in connection.findall(with_ns(XmlNs.CELLML, 'map_variables')):
                 connections_to_process.append(
@@ -704,7 +735,6 @@ class Transpiler(object):
         """https://www.w3.org/TR/MathML2/chapter4.html#contm.apply
         """
         result = self.transpile(node)
-
         if len(result) > 1:
             expression = result[0](*(result[1:]))
         else:
@@ -815,11 +845,17 @@ class Transpiler(object):
         operator taking qualifiers
         """
         def _wrapped_diff(x_symbol, y_symbol, evaluate=False):
+            if self._is_bool(x_symbol) or self._is_bool(y_symbol):
+                raise TypeError(f'Boolean not allowed in a Derivative: d{y_symbol} / d{x_symbol}')
             # if bound variable element <bvar> contains <degree>, argument x_symbol is a list,
             # otherwise, it is a symbol
             if isinstance(x_symbol, list) and len(x_symbol) == 2:
                 bound_variable = x_symbol[0]
-                order = int(x_symbol[1])
+                try:
+                    order = int(x_symbol[1])
+                except TypeError:
+                    raise TypeError(f'The degree of a derivative must be an int: d{y_symbol} / d{x_symbol}')
+
                 deriv = sympy.Derivative(y_symbol, bound_variable, order, evaluate=evaluate)
             # Otherwise, first degree derivative
             else:
@@ -888,6 +924,21 @@ class Transpiler(object):
                 for first, second in zip(expressions[:-1], expressions[1:]):
                     relations.append(sympy_relation(first, second))
                 return sympy.And(*relations)
+
+            if sympy_relation in (sympy.Ge, sympy.Le, sympy.Gt, sympy.Lt) and\
+                    any(map(self._is_bool, expressions)):
+                raise TypeError(f'Boolean not allowed in inequality: {expressions[0]} '
+                                f'{sympy_relation} {expressions[1]}')
+
+            if sympy_relation in (sympy.Eq, sympy.Ne) and any(map(self._is_bool, expressions))\
+                    and not all(map(self._is_bool, expressions)):
+                if any(map(lambda e: isinstance(e, sympy.Derivative), expressions)):
+                    raise TypeError(f'Boolean not allowed in inequality: {expressions[0]} '
+                                    f'{sympy_relation} {expressions[1]}')
+                else:
+                    logger.warning(f'Boolean used in part of (in)equality equation is this intentional?: '
+                                   f'{expressions[0]} {sympy_relation} {expressions[1]}')
+
             return sympy_relation(*expressions)
         return _wrapper_relational
 
@@ -905,9 +956,14 @@ class Transpiler(object):
 
         return handler
 
+    def _is_bool(self, expr):
+        """This function checks whether expr is a boolean (True/False).
+        """
+        return isinstance(expr, (sympy.logic.boolalg.BooleanTrue, sympy.logic.boolalg.BooleanFalse))
+
 
 # These MathML tags map directly to Sympy classes and don't require any extra handling
-SIMPLE_MATHML_TO_SYMPY_CLASSES = {
+_SIMPLE_MATHML_TO_SYMPY_CLASSES = {
     'abs': sympy.Abs,
     'and': sympy.And,
     'arccos': sympy.acos,
@@ -960,5 +1016,8 @@ SIMPLE_MATHML_TO_SYMPY_CLASSES = {
     'xor': sympy.Xor,
 }
 
+# MathML tags, to SYmpy mappings that can be changes (copy to allow original mapping to be accesible)
+SIMPLE_MATHML_TO_SYMPY_CLASSES = _SIMPLE_MATHML_TO_SYMPY_CLASSES.copy()
+
 # MathML relation elements that are n-ary operators
 MATHML_NARY_RELATIONS = {'eq', 'leq', 'lt', 'geq', 'gt'}
